---
title: Testing
tags:
- documentation
- tools
- tests
---

# Testing Drupal Code

This section describes everything we have to test our code from static 
code analysis to unit testing.

You also have to set up your environment with the tool [l3d](../docker/l3d/index.md). The composer plugin 
[Drupal Development Environment](../composer/plugin/drupal-dev-environment/index.md) does all the setup for you.

## Drupal Dependency Analysis

This tools provides a very useful functionality. You can check, if there are new versions of a module. We have the following
categories:

- Direct dependencies required in `composer.json`
- Transitive dependencies not required in `composer.json`

Usage:

```l3d
a test outdated
```

Note: **This can take a while depending on your installation.**

Provides an output something like this:

```l3d
Transitive dependencies not required in composer.json:
drupal/blazy                  2.16.0             2.27.0       Provides..
```


## Static Code Analysis

We have several tools to perform static code analysis.

- [PHP Code Sniffer](#php-code-sniffer)
- [PHPStan](#phpstan)
- [PHP Lint](#php-lint)
- [PHP LOC](#php-loc)
- [PHP Mess Detector](#php-mess-detector)
- [PHP Metrics](#php-metrics)
- [Stylelint](#stylelint)
- [Eslint](#es-lint)

### PHP Code Sniffer

PHP Code Sniffer performs a static code analysis with a specific Drupal profile. It detects
code smells like static calls. See [PHP Code Sniffer](https://github.com/squizlabs/PHP_CodeSniffer).

You can execute PHP CS for a specific Drupal contrib module:

```phpcs
a test phpcsmodule <contrib_module>
```

Now the entire code in that module will be analysed.

If you want to analyse a custom module, just do the following:

```phpcs
a test phpcsmodule ../custom/<custom_module>
```

To execute PHP Code Sniffer for a file or directory:

```phpcs
a test phpcs <file_ordirectory>
```

Whether it is a directory, it runs the analysis for everything under this directory
recursively.

Note: **There are many errors, which can be fixed automatically.** If you see something like this:

```phpcs
FILE: /var/www/html/web/modules/contrib/eca/src/Entity/Model.php
----------------------------------------------------------------------
FOUND 1 ERROR AFFECTING 1 LINE
----------------------------------------------------------------------
 37 | ERROR | [x] Additional blank lines found at end of doc comment
----------------------------------------------------------------------
PHPCBF CAN FIX THE 1 MARKED SNIFF VIOLATIONS AUTOMATICALLY
----------------------------------------------------------------------
```
You can just execute:

```phpcs
a test phpcs-fix <contrib_module>
```

Now th errors are gone, which saves a lot of time and effort.

### PHPStan

We use level 6 as default, which is pretty good and strict and provides a very clean code. You can adjust this in the
`phpstan.neon` file. See [PHPStan](https://phpstan.org/). It looks like this:

```phpstan
parameters:
  level: 6
```
It is recommended to have such a file in every module located in its root path.

You can execute PHP Stan for a specific Drupal contrib module:

```phpstan
a test phpstanmodule <contrib_module>
```

Now the entire code in that module will be analysed.

If you want to analyse a custom module, just do the following:

```phpstan
a test phpstanmodule ../custom/<custom_module>
```

To execute PHP Stan for a file or directory:

```phpstan
a test phpstan <file_ordirectory>
```

Whether it is a directory, it runs the analysis for everything under this directory
recursively.

### PHP Lint

You can execute [PHP Lint](https://github.com/overtrue/phplint) for a specific Drupal contrib module:

```phplint
a test phplintmodule <contrib_module>
```

Now the entire code in that module will be analysed.

If you want to analyse a custom module, just do the following:

```phplint
a test phplintmodule ../custom/<custom_module>
```

### PHP LOC

[PHP LOC](https://github.com/sebastianbergmann/phploc) prints several metrics about your code, like the average size of a class.

To execute PHP LOC on the console:

```phploc
a test phploc <file_ordirectory>
```

### PHP Mess Detector

[PHP Mess Detector](https://phpmd.org) is a more detailed analysis comparing to code sniffer. It 
uses several profiles like:

* cleancode
* design

This analysis detects also code smells like the complexity (e.g. number of
dependencies) of classes.

To execute PHP Mess Detector on the console:

```phpmd
a test phpmd <file_ordirectory> json <profile>
```

Whether it is a directory, it runs the analysis for everything under this directory
recursively.

### PHP Metrics

This useful tool provides more information as [PHP LOC](#php-loc). It adds for example:

- complexity
- bugs
- object-oriented programming stats
- coupling
- etc.

Note: **This can take a while depending on your module.**

You can execute [PHP Metrics](https://github.com/phpmetrics/PhpMetrics) for a specific Drupal contrib module:

```phpmet
a test phpmetricsmodule <contrib_module>
```

Now the entire code in that module will be analysed.

If you want to analyse a custom module, just do the following:

```phpmet
a test phpmetricsmodule ../custom/<custom_module>
```

### Stylelint

With `Stylelint` you can test and analyse `css` file in your module.

You first have to install and set up the toll with the following command:

```style
a test stylelintprepare
```

Now all you need is installed and set up. Execute the following command to run the test:

```style
a test stylelintmodule <contrib_module>
```

If you want to analyse a custom module, just do the following:

```style
a test stylelintmodule ../custom/<custom_module>
```

### ES Lint

This is another useful tool to test your `css` files.

```eslint
a test eslintmodule <contrib_module>
```

If you want to analyse a custom module, just do the following:

```eslint
a test eslintmodule ../custom/<custom_module>
```

## Unit/Kernel/Functional Testing

To execute the tests, you can use a single command:

```phpunit
a test phpunit <file_ordirectory>
```

All kind of Drupal tests will be executed.
Whether it is a directory, it runs the analysis for everything under this directory
recursively.

You also can execute the tests within your IDE.

First got to the settings of your IDE and add a new PHP interpreter. Select Docker and 
enter the form:

![Screenshot](/files/screenshots/settings_php_interpreter.png)

Use the option Docker Compose:

![Screenshot](/files/screenshots/settings_php_interpreter_docker.png)

Fill the following form like:

![Screenshot](/files/screenshots/settings_php_interpreter_new.png)

Use the server you created for [debugging](./debug.md).

The next step is to create a settings for your test framework. Remove all configuration
and add a new one for remote interpreter:

![Screenshot](/files/screenshots/settings_php_test_framework_new_config.png)



![Screenshot](/files/screenshots/settings_php_test_framework_new.png)

Set the autoloader and the configurations file.

Now you are ready to go. Open a directory or a single test file (or method) and click on the green
arrow:

![Screenshot](/files/screenshots/settings_php_test_execution.png)

You can also use the execution with the debugger.

### Code Coverage

First, activate the coverage option:

```debug
a debug coverage
```

Enter the projects to cover into your ```.lakedrops.yml``` file, like:

```yml
lakedrops-dev:
  coverage:
    - ../web/modules/contrib/bpmn_io
    - ../web/modules/contrib/camunda
    - ../web/modules/contrib/eca
    - ../web/modules/contrib/eca_entity_share
    - ../web/modules/contrib/eca_state_machine
    - ../web/modules/contrib/eca_tamper
```

This is just an example!

You have to execute the following command to write a new config file for testing:

```l3d
composer lakedrops:scaffold
```

Now you can execute every test including code coverage. To check, if it is
working properly, a new view for coverage should open in your IDE, like:

![Screenshot](/files/screenshots/settings_php_test_coverage_view.png)
